


def nicknames(namelist):
	"""Takes a list of names and trims off junk from the beginning
	and ends of the names to produce a set of reasonable nicknames
	that are relatively compact and easy to read.

	It returns a map from the names to the nicknames, along with
	the stuff trimmed off from the edges.
	"""

	left = ''
	right = ''
	ll = 1
	while ll <= len(namelist[0]):
		left = namelist[0][:ll]
		failed = False
		for nm in namelist:
			if not nm.startswith(left):
				failed = True
				break
		if failed:
			ll -= 1
			left = left[:ll]
			break
		ll += 1
	rr = 1
	while rr <= len(namelist[0]):
		right = namelist[0][-rr:]
		failed = False
		for nm in namelist:
			if not nm[ll:].endswith(right):
				failed = True
				break
		if failed:
			right = right[1:]
			rr -= 1
			break
		rr += 1
	map = {}
	for nm in namelist:
		map[nm] = nm[ll:len(nm)-rr]
	return (map, left, right)



def test():
	m, l, r = nicknames(['football.', 'fooberry.', 'fools gold.'])
	assert l == 'foo'
	assert r == '.'
	mi = m.items()
	mi.sort()
	assert mi == [('fooberry.', 'berry'), ('fools gold.', 'ls gold'), ('football.', 'tball')]
	assert l + m['football.'] + r == 'football.'

	m, l, r = nicknames(['abc', 'def'])
	assert l=='' and r==''
	assert m['abc'] == 'abc'
	assert l + m['abc'] + r == 'abc'

	m, l, r = nicknames(['abc'])
	assert l + m['abc'] + r == 'abc'

	m, l, r = nicknames(['val', 'str'])
	print m, l, r




if __name__ == '__main__':
	test()



"""This reads HTK feature vectors via the HList program."""


from gmisclib import die
from gmisclib import g_exec
import gpkimgclass
import numpy

def _parse_time(s):
	v, u = s.split()
	if u == 's':
		u = 1.0
	elif u == 'ms':
		u = 1e-3
	elif u == 'us':
		u = 1e-6
	elif u == 'ns':
		u = 1e-9
	elif u == 'ps':
		u = 1e-12
	else:
		die.die('Unrecognized time units: %s' % u)
	return float(v)*u


def read(fname):
	in_hdr = True
	in_os = False
	in_data = False
	os = 0
	ax1 = None
	hdr = {'CRPIX1': 1, 'CRPIX2': 1, 'CRVAL2': 0.0}
	xos = 'x'
	data = None
	hdr['_NAME'] = fname
	for line in g_exec.getiter_raw(None,
					['HList', '-h', '-o', '-i', '1', fname]
					):
		line = line.rstrip()
		if line == ' Observation Structure':
			in_os = True
			in_hdr = False
			continue
		elif in_os and line.startswith(' Samples:'):
			in_os = False
			in_data = True
			data = numpy.zeros((hdr['NAXIS2'], hdr['NAXIS1']))
			continue
		elif line.startswith('----- END'):
			in_data = False

		if in_hdr:
			k, v = line.strip().split(':', 1)
			k = k.strip()
			v = v.strip()
			if k == 'Num Comps':
				hdr['NAXIS1'] = int(v)
			elif k == 'Num Samples':
				hdr['NAXIS2'] = int(v)
			elif k == 'Sample Period':
				hdr['CDELT2'] = _parse_time(v)
			elif k == 'File Format':
				hdr['_FILETYPE'] = v
			else:
				hdr[k] = v
		elif in_os:
			if not line.startswith(' ') and ':' in line:
				xos, tt = line.split(':', 1)
				xos = xos.strip()
				tt = tt.strip()
				os = 0
			else:
				tt = line.strip()
			hdr['TTYPE%d' % (os+1)] = '%s:%s' % (xos, tt)
			os += 1
		elif in_data:
			if ':' in line:
				os = 0
				ax1, line = line.split(':')
				ax1 = int(ax1)
			data[ax1,os] = float(line)
			os += 1
	return gpkimgclass.gpk_img(hdr, data)

if __name__ == '__main__':
	# /home/mace/jyuan/irr_pdur/plp_ip
	import sys
	import pylab
	d = read(sys.argv[1])
	numpy.subtract(d.d, numpy.average(d.d, axis=0), d.d)
	numpy.divide(d.d, numpy.std(d.d, axis=0), d.d)
	pylab.imshow(d.d.transpose(),
			interpolation='nearest', origin='lower',
			aspect = 'normal',
			)
	pylab.show()



#!/usr/bin/env python

"""When run as a script:
python ~/lib/wavio.py [-g gain] -wavin|-wavout infile outfile
Reads or writes .wav files from any format supported by
gpkimgclass.py .

As a library, allows reading of class gpk_img data from a .WAV
file, and writing class gpk_img to a .WAV file.
"""
import wave
import numpy
import gpkimgclass
from gmisclib import wavio


if __name__ == '__main__':
	import sys
	arglist = sys.argv[1:]
	gain = None
	if len(arglist)==0:
		print __doc__
		sys.exit(1)
	if arglist[0] == '-g':
		arglist.pop(0)
		gain = float(arglist.pop(0))
	if arglist[0] == '-wavin':
		x = wavio.read(arglist[1])
		if gain is not None:
			numpy.multiply(x.d, gain, x.d)
		x.write(arglist[2])
	elif arglist[0] == '-wavout':
		x = gpkimgclass.read(arglist[1])
		rxd = numpy.ravel(x.d)
		mxv = max( rxd[numpy.argmax(rxd)], -rxd[numpy.argmin(rxd)] )
		print "# max=", mxv
		if gain is None:
			gain = 32000.0/mxv
		wavio.write(x, arglist[2], gain)
	else:
		print __doc__
		sys.exit(1)



"""Support module for multivariance.py"""

from gmisclib import gpkmisc
from gmisclib import Num
from gmisclib import die


BAYES_PRIOR_SPACE = 0	# 1 for a flat prior in sigma,
HUGE = 1e30

class QuadraticNotNormalizable(ValueError):
	def __init__(self, s=''):
		ValueError.__init__(self, s)





def vec_inv_variance(start):
	vv = gpkmisc.vec_variance(start)
	assert Num.alltrue(Num.greater(vv, 0.0))
	return 1.0/vv


def diag_inv_variance(start):
	return gpkmisc.make_diag(vec_inv_variance(start))



class modeldesc:
	__doc__ = """Virtual base class for a description of a model
			of a particular size."""

	LF = (1.0/3.0)**2

	def __init__(self, ndim):
		assert ndim > 0
		self.dim = ndim

	def ndim(self):
		"""This returns the dimensionality of the data."""
		return self.dim

	def modeldim(self):
		"""This gives the dimensionality of the model,
		i.e. the number of parameters required to specify
		the means and covariance matrix(ces)."""
		raise RuntimeError, 'Virtual method'

	def unpack(self, prms):
		"""This returns some subclass of model."""
		raise RuntimeError, 'Virtual method'

	def new(self, mu, invsigma):
		"""Creates a model that contains data."""
		raise RuntimeError, 'Virtual method'

	def start(self, dataset):
		"""Selects a random starting point from the dataset."""
		raise RuntimeError, 'Virtual method'



class model_with_numbers:
	__doc__ = """Virtual base class for adding in the functions
		you need when you create a model with known parameters
		(like mu and sigma)."""

	def __init__(self, details, bias):
		"""Bias is an overall shift of the log probability up or
		down.  In a classifier, it is used to bias things toward
		one class or another."""
		self.desc = details
		self._offset = None
		self.bias = bias

	def logp(self, datum):
		raise RuntimeError, 'Virtual method'

	def pack(self):
		"""Returns a vector of parameters."""
		raise RuntimeError, 'Virtual method'

	def ndim(self):
		return self.desc.ndim()

	def unpack(self, prms):
		return self.desc.unpack(prms)

	def new(self, mu, invsigma):
		return self.desc.new(mu, invsigma)

	def start(self, dataset):
		return self.desc.start(dataset)

	def offset(self):
		if self._offset is None:
			self.addoff()
		return self._offset

	def addoff(self):	# Should not be called if _offset is not None
		raise RuntimeError, 'Virtual Function'






def _q_addoff(self):	# Will not be called if _offset is not None
	phasespacefac = BAYES_PRIOR_SPACE+1
	determinantfac = 1 - phasespacefac
	trace = Num.sum(Num.diagonal(self.invsigma))
	if trace <= 0.0:
		raise QuadraticNotNormalizable, "Input trace is nonpositive."
	try:
		self.ev = Num.LA.Heigenvalues(self.invsigma)
	except Num.LA.LinAlgError, x:
		die.warn('While computing the volume of the probability distribution: %s' % str(x))
		raise QuadraticNotNormalizable, x
	if not Num.alltrue(self.ev > 0.0):
		raise QuadraticNotNormalizable, "Some eigenvalues are zero or negative."
	assert abs(Num.sum(self.ev)-trace) < 1e-10 + 1e-6*trace, "Bad Eigenvalues run: trace not matched: %g to %g" % (trace, Num.sum(self.ev))
	assert Num.sum(self.ev) > 0.0, "Input trace is nonpositive."
	self._offset = Num.sum(Num.log(self.ev))*determinantfac



def _d_addoff(self):	# Will not be called if _offset is not None
	phasespacefac = BAYES_PRIOR_SPACE+1
	determinantfac = 1 - phasespacefac
	self.ev = Num.array(self.invsigma, copy=True)
	if not Num.alltrue(self.ev > 0.0):
		raise QuadraticNotNormalizable, "Some eigenvalues are zero or negative."
	self._offset = Num.sum(Num.log(self.ev))*determinantfac






"""Module to parse and create lines in "a=v;" format.
Normally, the entry points are parse() and concoct().
"""

from gmisclib import gpk_writer
from gmisclib import g_encode
from gmisclib import die

__version__ = "$Revision: 1.21 $"

BadFormatError = g_encode.BadFormatError
_default_encoder = g_encode.encoder()



fwd = _default_encoder.fwd
back = _default_encoder.back


def parse(s, sep=';'):
	"""Parses a line in a=v; a=v form into a dictionary.
	The line may be terminated by a '#' followed by an arbitrary
	comments.    Both attributes and values are assumed to be
	encoded with the default L{g_encode} encoder.  (However, the
	comment is not assumed to be encoded.)

	@param s: the string to be parsed.
	@param sep: (optional) lets you choose the character
		that separates attribute-value pairs.
	@return: dictionary containing a=v pairs.   One special
		entry for comments: "_COMMENT".
	"""
	rv = {};
	if '#' in s:
		ss, c = s.split('#', 1)
		cs = c.strip()
		if cs:
			rv['_COMMENT'] = cs
		if not ss:
			return rv
	else:
		ss = s
	for av in ss.split(sep):
		try:
			a, v = av.split('=', 1)
		except ValueError:	# need more than 1 value to unpack
			# It's OK if it is entirely whitespace...
			if av.strip():
				raise BadFormatError, 'no equals sign in "%s"' % str(av)
		else:
			rv[a.strip()] = back(v.strip())
	return rv


def concoct(s):
	"""Converts a dictionary to a line in the form a=v;a=v;#comment.
	It returns a string.   The entries in the dictionary will be
	converted to a string representation by fwd()."""
	c = None
	l = []
	kvl = s.items()
	kvl.sort()
	for (k,v) in kvl:
		if k == '_COMMENT':
			c = str(v)
		else :
			l.append('%s=%s;' % (k, fwd(v)));
	if c is not None:
		l.extend( [ '#', c ] )
	return ' '.join(l)


def read(fd, sep=';', line=None):
	"""Read a file in a=v; format.
	If line is set, put the line number into a slot with the specified name.
	@return: (data, comments) where data=[{a:v, ...}, ...] and comments=[str, ...]
	@rtype: tuple(list(dict), list(str))
	"""
	comments = []
	data = []
	line_num = 0
	# while True:
	for l in fd:
		line_num += 1
		# l = fd.readline()
		# if l == '':
			# break
		if l.startswith('#'):
			c = l[1:].strip()
			if c != '':
				comments.append(c)
			continue
		if not l.endswith('\n'):
			die.warn('avio.read(): ignoring line without newline.')
			continue
		l = l.strip()
		if l == '':
			continue
		tmp = parse(l, sep)
		if line is not None:
			tmp[line] = line_num
		data.append( tmp )
	return (data, comments)


def test1():
	d = {}
	d['test'] = 'george'
	d['fred'] = 'rrr;;#='
	d['mt'] = ''

	o = parse(concoct(d) + " # comment")

	assert o['test'] == 'george'
	# print "o['fred']=", o['fred']
	assert o['fred'] == 'rrr;;#='
	assert o['mt'] == ''


class writer(gpk_writer.writer):
	__doc__ = """Write a file in a=v; format.
			"""

	def comment(self, comment):
		"""Add a comment to the data file."""
		self.fd.write("# %s\n" % comment)

	def header(self, k, v):
		"""NOTE: this is not a fully general function.
		Keys may not have spaces or equals signs in them.
		Note also that reading of headers from a=v; files
		is not supported.
		"""
		self.fd.write("# %s = %s\n" % (k, fwd(v)))
	
	def __init__(self, fd):
		gpk_writer.writer.__init__(self, fd)

	def datum(self, data_item):
		self.fd.writelines( [ concoct(data_item), '\n'] )



def read_hdc(fd, sep=';', line=None):
	"""This emulates fiatio.read().
	@return: (header, data, comments) where data=[{a:v, ...}, ...] and comments=[str, ...]
	@rtype: tuple(dict, list(dict), list(str))
	"""
	d, c = read(fd, sep=sep, line=line)
	if d:
		h = d[0].copy()
	else:
		h = {}
	for datum in d:
		for (k, v) in h.items():
			if k in datum and datum[k]!=v:
				del h[k]
	hk = h.keys()
	for datum in d:
		for k in hk:
			del d[k]
	return (h, d, c)



def test2():
	import StringIO
	fd = StringIO.StringIO()
	w = writer(fd)
	w.comment('foo')
	w.datum( {'a':'z', 'q':'u'} )
	w.close()
	fd.seek(0)
	d, c = read(fd)
	assert c == ['foo']
	assert len(d) == 1
	assert d[0]['a'] == 'z'
	assert d[0]['q'] == 'u'


if __name__ == '__main__' :
	assert back(fwd('hello \n!@;==')) == 'hello \n!@;=='
	test1()
	test2()
	print "OK: passed tests"
	# print concoct({'x': "a=b;"})



"""Defines array_window class."""

__version__ = "$Revision: 1.4 $"

class array_window:
	__doc__ = """Lets you reference 1-d array items relative to an arbitrary zero."""

	def __init__(self, array, zero):
		assert array is not None
		assert len(array) > 0
		assert zero >= 0 and zero < len(array)
		self.d = array
		self.zero = zero

	def __setattr__(self, name, value):
		if self.__dict__.has_key(name):
			raise AttributeError, "Can't re-set " + name
		if name == 'd' or name == 'zero':
			self.__dict__[name] = value
		else:
			raise AttributeError, "Can't set any attributes"

	def __delattr__(self, name):
		raise(AttributeError)

	def __getitem__(self, index):
		i = self.zero + index
		if i < 0:
			raise IndexError, 'negative index'
		return self.d[i]

	def __setitem__(self, index, value):
		index += self.zero
		if index < 0:
			raise IndexError, 'negative index'
		self.d[index] = value

	def __getslice__(self, i, j):
		i += self.zero
		j += self.zero
		if i < 0:
			i = 0
		if j < 0:
			j = 0
		return self.d[i, j]

	def __setslice__(self, i, j, sequence):
		i += self.zero
		j += self.zero
		if i < 0:
			i = 0
		if j < 0:
			j = 0
		self.d[i:j] = sequence

	def __str__(self):
		if self.zero <= 0:
			s = "*" + (-self.zero)*"." + str(self.d)
		elif self.zero > len(self.d):
			s = str(self.d) + (self.zero-len(self.d))*"." + "*"
		else:
			s = str(self.d[:self.zero]) + "*" + str(self.d[self.zero:])
		return "<array_window at %d of %d:" % (self.zero, len(self.d)) + s + ">"

	def __repr__(self):
		return self.__str__()

	def __len__(self):
		"""Of course, one can't assume that one can say x[i] for i in range(len(x))."""
		return len(self.d)


def test():
	x = [0, 2, 3, 4, 5, 6, 7, 8, 9]
	y = array_window(x, 3)
	assert y[0] == x[3]
	assert y[2] == x[5]

if __name__ == '__main__' :
	test()
	print "OK: passed tests"



#!/usr/bin/env python

from gmisclib import Num
from gmisclib import blue_data_selector
from gmisclib import die



def _mswap(sw1, sw2, m):
	"""Swap rows and columns in matrix m.
	Produces a copy of the matrix.
	@param m: matrix
	@param sw1: how to re-arrange rows(columns?)
	@param sw2: how to re-arrange columns(rows?)
	"""
	assert sw1.shape == (m.shape[0],)
	assert sw2.shape == (m.shape[1],)
	m1 = Num.take(m, sw1, axis=0)
	return Num.take(m1, sw2, axis=1)


def _lswap(swv, m):
	"""Swap labels in list m, according to mapping swv.
	Produces a new list.
	"""
	assert swv.shape == (len(m),)
	return [ m[swv[i]] for i in range(len(m)) ]



class diagfom(object):
	"""A class that defines a figure-of-merit for a matrix.
	"""
	def __init__(self, n, sign=1):
		"""@param sign: if sign=1, the f-o-m returned by L{eval}
			will be minimal when
			the most positive matrix elements are on the diagonal.
			If sign=-1, you'll get minimal fom with the
			the most positive elements off diagonal.
		@type sign: int
		@param n: dimension of matrix.
		@type n: int
		"""
		ar = Num.arrayrange(n)
		self.weight = sign*Num.absolute(ar[:,Num.NewAxis] - ar[Num.NewAxis,:])

	def eval(self, swv, m):
		"""Evaluate the figure of merit for a matrix m with the
		specified swap vector.
		@param m: matrix
		@param swv: swap vector
		@return: figure of merit
		@rtype: int or float (according to the type of m).
		"""
		m2 = _mswap(swv, swv, m)
		# return Num.sum(Num.ravel(m2*self.weight))
		return Num.sum(m2*self.weight)


class diagfom2(object):
	def __init__(self, n1, n2):
		a1 = Num.arrayrange(n1)
		a2 = Num.arrayrange(n2)
		self.weight = Num.absolute(a1[:,Num.NewAxis]/float(n1-1) - a2[Num.NewAxis,:]/float(n2-1))

	def eval(self, sw1, sw2, m):
		m2 = _mswap(sw1, sw2, m)
		# return Num.sum(Num.ravel(m2*self.weight))
		return Num.sum(m2*self.weight)


class blockfom(object):
	def __init__(self, n1, n2):
		pass

	def eval(self, sw1, sw2, m):
		m2 = _mswap(sw1, sw2, m)
		o = 0.0
		# o += Num.sum(Num.ravel(Num.absolute(m2[1:,:] - m2[:-1,:])))
		# o += Num.sum(Num.ravel(Num.absolute(m2[:,1:] - m2[:,:-1])))
		o += Num.sum(Num.absolute(m2[1:,:] - m2[:-1,:]))
		o += Num.sum(Num.absolute(m2[:,1:] - m2[:,:-1]))
		return o


def symm_swap_toward_minimal_fom(m, fom, maxtries):
	n = m.shape[0]
	assert m.shape == (n, n)
	swv = Num.arrayrange(n)
	of = fom.eval(swv, m)
	sincelast = 0
	bds = blue_data_selector.bluedata( [ (i,j) for i in range(n) for j in range(n) if i!=j ] )
	if maxtries is None:
		maxtries = 2*n*n
	for (i,j) in bds:
		oswv = Num.array(swv, Num.Int, copy=True)
		swv[i], swv[j] = (swv[j], swv[i])
		nf = fom.eval(swv, m)
		if nf < of:
			of = nf
			sincelast = 0
		else:
			swv = oswv
			sincelast += 1
		if sincelast > maxtries:
			break
	return swv


def swap_toward_minimal_fom(m, fom, maxtries):
	n1, n2 = m.shape
	assert m.shape == (n1, n2)
	sw1 = Num.arrayrange(n1)
	sw2 = Num.arrayrange(n2)
	of = fom.eval(sw1, sw2, m)
	sincelast = 0
	bd1 = blue_data_selector.bluedata( [ (i,j) for i in range(n1) for j in range(n1) if i!=j ] )
	bd2 = blue_data_selector.bluedata( [ (i,j) for i in range(n2) for j in range(n2) if i!=j ] )
	if maxtries is None:
		maxtries = 2*(n1*n1 + n2*n2)
	else:
		maxtries = 2*maxtries
	while True:
		i, j = bd1.pick(1)[0]
		osw1 = Num.array(sw1, Num.Int, copy=True)
		sw1[i], sw1[j] = (sw1[j], sw1[i])
		nf = fom.eval(sw1, sw2, m)
		if nf < of:
			die.info('fom=%g->%g; sincelast=%d' % (of, nf, sincelast))
			of = nf
			sincelast = 0
		else:
			sw1 = osw1
			sincelast += 1

		i, j = bd2.pick(1)[0]
		osw2 = Num.array(sw2, Num.Int, copy=True)
		sw2[i], sw2[j] = (sw2[j], sw2[i])
		nf = fom.eval(sw1, sw2, m)
		if nf < of:
			of = nf
			die.info('fom=%g->%g; sincelast=%d' % (of, nf, sincelast))
			sincelast = 0
		else:
			sw2 = osw2
			sincelast += 1

		i, j = bd1.pick(1)[0]
		osw1 = Num.array(sw1, Num.Int, copy=True)
		sw1[i], sw1[j] = (sw1[j], sw1[i])
		i, j = bd2.pick(1)[0]
		osw2 = Num.array(sw2, Num.Int, copy=True)
		sw2[i], sw2[j] = (sw2[j], sw2[i])
		nf = fom.eval(sw1, sw2, m)
		if nf < of:
			of = nf
			die.info('fom=%g->%g; sincelast=%d' % (of, nf, sincelast))
			sincelast = 0
		else:
			sw1 = osw1
			sw2 = osw2
			sincelast += 1

		if sincelast > maxtries:
			break
	return (sw1, sw2)



def swap_toward_diag(m, lbls, maxtries=None):
	"""Swap the rows and columns of a matrix to bring it closer to a diagonal
	matrix: i.e. large entries on the main diagonal and small entries
	away from the main diagonal.    Rows and columns are swapped together,
	so that the ordering or rows will match the ordering of columns.
	"""
	
	fom = diagfom(len(lbls))
	swv = symm_swap_toward_minimal_fom(Num.absolute(m), fom, maxtries)
	return ( _mswap(swv, swv, m), _lswap(swv, lbls) )


def swap_toward_diag2(m, lbl1, lbl2, maxtries=None):
	"""Swap the rows and columns of a matrix to make it roughly
	diagonal:
	i.e. large entries on the main diagonal and small entries
	away from the main diagonal.   Note that this will work
	even if the matrix is not square.
	"""
	fom = diagfom2(len(lbl1), len(lbl2))
	sw1, sw2 = swap_toward_minimal_fom(Num.absolute(m), fom, maxtries)
	return ( _mswap(sw1, sw2, m), _lswap(sw1, lbl1), _lswap(sw2, lbl2) )


def pos_near_diag2(m, lbl1, lbl2, maxtries=None):
	fom = diagfom2(len(lbl1), len(lbl2))
	sw1, sw2 = swap_toward_minimal_fom(m, fom, maxtries)
	return ( _mswap(sw1, sw2, m), _lswap(sw1, lbl1), _lswap(sw2, lbl2) )


def neg_near_diag2(m, lbl1, lbl2, maxtries=None):
	fom = diagfom2(len(lbl1), len(lbl2))
	sw1, sw2 = swap_toward_minimal_fom(-m, fom, maxtries)
	return ( _mswap(sw1, sw2, m), _lswap(sw1, lbl1), _lswap(sw2, lbl2) )


def swap_toward_blocks(m, lbl1, lbl2, maxtries=None):
	"""Swap rows and columns of a matrix to bring it closer to a block
	form, where similar values occur together in blocks.
	"""
	fom = blockfom(len(lbl1), len(lbl2))
	sw1, sw2 = swap_toward_minimal_fom(m, fom, maxtries)
	return ( _mswap(sw1, sw2, m), _lswap(sw1, lbl1), _lswap(sw2, lbl2) )







import inspect
import sys
import traceback

debug = True	# Set to False to disable dbg() messages.
compress = True

_name = sys.argv[0]
_ni = _name.rfind('/')
if _ni >=0 and _ni != len(_name)-1:
	_name = _name[_ni+1:]

logfd = sys.stderr

_last = ()
_counter = 0


def note(name, val):
	stack = inspect.stack()
	uplocal = stack[1][0].f_locals
	stack = None
	if not uplocal.has_key('__note'):
		uplocal['__note'] = {}
	uplocal['__note'][name] = val
	uplocal = None


def show(name):
	stack = inspect.stack()
	uplocal = stack[1][0].f_locals
	stack = None
	if not uplocal.has_key('__show'):
		uplocal['__show'] = []
	uplocal['__show'].append(name)
	uplocal = None



def _notes(n):
	# __note = {'WHOOPS': 'WHOOPS'}
	stack = inspect.stack()
	o = []
	for level in range(len(stack)-1, n, -1):
		uplocal = stack[level][0].f_locals
		if '__show' in uplocal:
			for k in uplocal['__show']:
				if k in uplocal:
					o.append( (k, uplocal[k]) )
			del uplocal['__show']
		
		if '__note' in uplocal:
			o.extend( uplocal['__note'].items() )
			del uplocal['__note']
		
		del uplocal
	del stack
	o.sort()

	# Delete duplicate values.  This can happen if
	# a variable is noted at two different levels.
	i = 1
	while i < len(o):
		if o[i] == o[i-1]:
			del o[i]
		else:
			i += 1

	return o


def _dumpmem(n):
	logfd.flush()
	if logfd is not sys.stderr:
		sys.stderr.flush()
	sys.stdout.flush()
	for (k, v) in _notes(n+1):
		logfd.write('#NOTE: %s = %s\n' % (k, v))
	logfd.flush()


def _display(prefix, name, text, level):
	global _counter
	global _last
	if compress and _last == (prefix, name, text):
		_counter += 1
		return
	elif _counter > 0:
		logfd.write('#INFO: : preceeding item repeated %d times.\n' % (_counter+1))
		_counter = 0
		_last = ()
	_dumpmem(level+1)
	logfd.write('#%s: %s: %s\n' % (prefix, name, text))
	_last = (prefix, name, text)
	logfd.flush()


def die(s):
	"""Output a fatal error message and terminate."""
	e = 'ERR: %s: %s' % (_name, s)
	exit(1, e)


def warn(s):
	"""Output a non-fatal warning."""
	# __note = {'WHOOPS': 'WHOOPS'}
	_display('WARN', _name, s, 1)


def info(s):
	"""Output useful information."""
	global _q
	_display('INFO', _name, s, 1)


def catch(extext=None):
	"""Call this inside an except statement.
	It will report the exception and any other information it has."""
	type, value, tback = sys.exc_info()
	if extext is None:
		extext = "die.catch: exception caught.\n"
	_dumpmem(1)
	traceback.print_exception(type, value, tback)
	sys.stderr.flush()
	sys.stdout.flush()
	

def catchexit(extext=None, n=1, text=None):
	"""Call this inside an except statement.  It will report
	all information and then exit."""
	type, value, tback = sys.exc_info()
	if extext is None:
		extext = "die.catch: exception caught.\n"
	_dumpmem(1)
	traceback.print_exception(type, value, tback)
	sys.stderr.flush()
	sys.stdout.flush()
	if text is not None:
		sys.stdout.write('%s\n' % text)
		sys.stdout.flush()
		logfd.write('%s\n' % text)
		logfd.flush()
	sys.exit(n)


def dbg(s):
	"""Output debugging information, if debug is nonzero."""
	if debug:
		_display('DBG', _name, s, 1)


def exit(n, text=None):
	"""Exit, after dumping accumulated messages."""
	_dumpmem(1)
	if text is not None:
		sys.stdout.write('%s\n' % text)
		sys.stdout.flush()
		logfd.write('%s\n' % text)
		logfd.flush()
	sys.exit(n)


def get(key):
	nl = _notes(0)
	for (k, v) in nl:
		if k == key:
			return v
	raise KeyError, key



if __name__ == '__main__':
	debug = 1
	info("You should see a debug message next.")
	dbg("This is the debug message.")
	debug = 0
	note("gleep", "oldest note")
	note("foo", "bar")
	note("foo", "fleep")
	note("foo", "foo")
	note("farf", "newest note")
	info("You should not see a debug message next.")
	dbg("This is the debug message you shouldn't see.")
	warn("This is a warning.")
	warn("This is a warning.")
	warn("This is a warning.")
	info("You should have seen three identical warnings just above.")
	die("This is the end.")



"""Viterbi search"""

import Numeric




def path(nodecost, linkcost):
	# nodecost[t,j]  at time t, pitch=j
	# linkcosts[j1, j2]   cost to go from (t,j1) to (t+1,j2)
	#    where j=pitch

	T = nodecost.shape[0]
	N = nodecost.shape[1]



	cost = Numeric.zeros((N,), Numeric.Float)

	bestpathto = []
	for j in range(N):
		bestpathto.append([j])
		cost[j] = nodecost[0,j]

	for t in range(1,T):
		nbp = []
		ncost = Numeric.zeros((N,), Numeric.Float)
		for j in range(N):
			cc = cost + linkcost[:,j]
			o = Numeric.argmin(cc)
			ncost[j] = cc[o] + nodecost[t,j]
			nbp.append(bestpathto[o] + [j])
		bestpathto = nbp
		cost = ncost

	jj = Numeric.argmin(cost)

	return (cost[jj], bestpathto[jj])



def test1():
	T = 10
	N = 7
	noc = Numeric.zeros((T,N), Numeric.Float) + 5
	noc[:,3] = 1
	linkc = Numeric.zeros((N,N), Numeric.Float) + 2

	c, p = path(noc, linkc)
	assert abs(c - (10*1 + 9*2)) < 0.0001
	assert p == [3]*T

def test2():
	T = 10
	N = 7
	noc = Numeric.zeros((T,N), Numeric.Float) + 5
	for i in range(T):
		noc[i,i%N] = 1
	linkc = Numeric.zeros((N,N), Numeric.Float) + 2

	c, p = path(noc, linkc)
	assert abs(c - (10*1 + 9*2)) < 0.0001
	assert p == [0,1,2,3,4,5,6,0,1,2]


def test3():
	T = 10
	N = 7
	noc = Numeric.zeros((T,N), Numeric.Float) + 1
	noc[0,0] = 0
	linkc = Numeric.zeros((N,N), Numeric.Float) + 2
	for i in range(N):
		linkc[i,(i+1)%N] = 1

	c, p = path(noc, linkc)
	assert abs(c - (0+9*1 + 9*1)) < 0.0001
	assert p == [0,1,2,3,4,5,6,0,1,2]

if __name__ == '__main__':
	test1()
	test2()
	test3()



#!/usr/bin/env python

"""This program takes files named *.q in the current directory, and converts them to HTML.
To do that, it looks for a python script called 'headfoot.py' or '../headfoot.py' or ... .
This script needs to define one variable ( server ) and two functions (header and footer).

The .q files have the following format:

TITLE title string
OTHER_HEADER_KEYWORD header info

P
	Text is 3>5 ?
	A ref="foo.html" No!
		More text inside the link.
	IMG ref="foo.gif"
	UL
		LI List item
		LI Another List item


The header info is separated from the body with a blank line.
HTML tags are always the first thing on a line.
Line continuations and enclosures are indicated with indentation.
Tags automatically close when the indentation gets smaller.

The PRE tag is the only exception.
You have to close it with a line beginning '/PRE'.
No indenting is needed inside a PRE /PRE pair.

The user needs to define one thing:
the address of the web server.
This address is prepended to
all hyperlinks and image references that aren't absolute.

All the header information gets put into a dictionary, and is
passed into the header() and footer() functions.
That dictionary has 'filename' and '_server_root' entries added.

The user can also define three things:

DEFAULT_HEADER, a dictionary passed to the header() and footer functions().
Useful things to put in DEFAULT_HEADER are
	'lang': 'en'	# appears in the <html lang="en"> tag
	'stylesheet': url becomes a link to a stylesheet: <link rel="stylesheet" type="text/css" href="url">
	Anything in the form M.x:y becomes a <meta name="x" content="y">
	M.description and M.keywords are useful.
	Anything in the form HTTP.x:y becomes a <meta http-equiv="x" content="y">

Header(thd, hinfo) takes two arguments, a file descriptor
to which it should write the HTML, and a dictionary of header
information.

Footer(thd, hinfo) takes the same arguments.

See _header() and _footer() below, for examples.
"""

import os
import re
import die
import time
import sets
import urlparse
import g_pipe

XML = 0

POINTTAG = 1
_anytag = {'P':0, 'STYLE': 0,
		'UL':0, 'OL':0, 'DL':0,
		'LI':0, 'DT': 0, 'DD': 0,
		'PRE':'SPECIAL',
		'TABLE':0, 'TR':0, 'TD':0,
		'CAPTION':0,
		'TBODY':0, 'THEAD':0, 'TFOOT':0,
		'FORM':0, 'INPUT':1, 'SELECT':0, 'OPTION':0, 'TEXTAREA':0,
		'A':0, 'NOP': 0,
		'H1':0, 'H2':0, 'H3':0, 'H4':0,
		'EM':0, 'STRONG':0, 'CITE':0, 'DFN':0,
		'CODE':0, 'SAMP':0, 'KBD':0, 'VAR':0, 'ABBR':0, 'ACRONYM':0,
		'OBJECT': 0, 'DIV': 0, 'SPAN': 0,
		'FONT': 0,
		'IMG': POINTTAG, 'BR': POINTTAG, 'HR': POINTTAG
		}

_ck = re.compile(r'\b(H1|PRE|P|TITLE|IMG)\b', re.IGNORECASE)

def _checkfile(f):
	"""Look at the beginning of a file to see if is plausibly input for this program."""
	fd = open(f, "r")
	for l in fd.readlines()[:10]:
		if _ck.match(l):
			return 1
	return 0



def qfiles(dir):
	o = []
	minname = re.compile('^.*[.]q$')
	l = os.listdir(dir)
	for t in l:
		ts = t.strip()
		x = minname.match(ts)
		if not x:
			continue
		if not _checkfile(ts):
			continue
		o.append(ts)
	return o




def av(k, v):
	if not v.startswith('"') or not v.endswith('"'):
		v = '"%s"' % v
	return "%s=%s" % (k, v)


LL = 60
TABW = 8

DEFAULT_TAG = 'P'
DEFAULT_INDENT = -1

def measure_indent(s):
	indent = 0
	i = 0
	if s.isspace():
		return (None, '')

	while i<len(s) and s[i].isspace():
		if s[i] == ' ':
			indent += 1
		elif s[i] == '\t':
			indent = TABW*((indent+TABW)/TABW)
		elif s[i] == '\r' or s[i] == '\n':
			return (None, '')
		else:
			raise ValueError, "Unrecognized whitespace: %d" % ord(s[i])
		i += 1
	return indent


def prepare_text(l):
	l = l.strip()
	if l.startswith('%'):
		l = l[1:].lstrip()
	return l


def starts_with_a_tag(l):
	a = l.split(None, 1)
	if len(a) == 0:
		return False
	elif len(a) == 1:
		return _anytag.has_key(a[0])
	else:
		return _anytag.has_key(a[0]) and '=' in a[1]


_eqtoken = re.compile("""\s*([a-zA-Z0-9_:]+)\s*=\s*(("[^"]*")|(\S+))\s*""")
def tokenize(l):
	# print "TOKENIZE:", l.strip()
	a = l.split(None, 1)
	tag = a[0]
	args = {}
	if len(a) > 1:
		s = a[1]
		while True:
			m = _eqtoken.match(s)
			if not m:
				break
			args[m.group(1)] = m.group(2)
			s = s[m.end():]
		txt = s
	else:
		txt = ''
	die.info("TOKENIZED:%s %s %s" % (tag, args, txt))
	return (tag, args, txt)


class lineC:
	def __init__(self, l):
		self.indent = measure_indent(l)
		l = l.strip()
		if starts_with_a_tag(l):
			self.tag, self.args, self.txt = tokenize(l)
		else:
			self.tag = None
			self.args = None
			self.txt = prepare_text(l)

	def getarg(self, k, defv):
		return self.args.get(k, defv)




def format_dict(d, drop=None):
	o = []
	if drop is None:
		for (k, v) in d.items():
			o.append(av(k, v))
	else:
		for (k, v) in d.items():
			if not k.startswith(drop):
				o.append(av(k, v))
	return o


def dequote(a):
	if len(a) > 1 and a[0]=='"' and a[-1]=='"':
		return a[1:-1]
	return a

def urlq(s):
	return s


def prepend(a, b):
	"""Process a URL.  Pathnames are prepended with the server root.
	Complete URLs are untouched."""

	# print 'JOIN', a['_abs_root'] + '/' + a['_cwd'] + '/', b
	return '"%s"' % urlparse.urljoin(urlq(a['_abs_root'] + '/' + a['_cwd'] + '/'),
					urlq(dequote(b)))



def process_tag(d, od, hinfo, eol):
	references = []

	# print "Tag=", d.tag
	if d.tag == 'IMG':
		d.args['src'] = prepend(hinfo, d.args['ref'])
		references = [ d.args['src'] ]
		del d.args['ref']
	elif d.tag == 'OBJECT':
		d.args['data'] = prepend(hinfo, d.args['ref'])
		references = [ d.args['data'] ]
		del d.args['ref']
	elif d.tag == 'A' and 'ref' in d.args:
		d.args['href'] = prepend(hinfo,  d.args['ref'])
		references = [ d.args['href'] ]
		del d.args['ref']

	args = format_dict(d.args, '_')

	if XML:
		pointclose = ([], ['/'])[ _anytag[d.tag]==POINTTAG ]
	else:
		pointclose = []

	od.write('<%s>%s%s' % (' '.join([d.tag] + args + pointclose), d.txt, eol))

	if not _anytag[d.tag] == POINTTAG:
		return (references, d.tag)
	return (references, None)


_escape = {'~': '&nbsp;', '>':'&gt;', '<':'&lt;', '&':'&amp;',
		'"': '&quot;'
		}
_epat = '&[0-9a-zA-Z]{,5};|' + ( '|'.join(_escape.keys()) )
# print "EPAT=", _epat
_ere = re.compile(_epat)
def _esc1(x):
	"""Converts a character in a MatchObject to a &xx; (HTML) escape sequence"""
	mstring = x.group(0)
	if len(mstring)==1:
		return _escape[mstring]
	return mstring



def escape(s):
	return _ere.sub(_esc1, s)


def get_logical_line(lines):
	while 1:
		if len(lines) == 0:
			break
		o = lines.pop(0)
		if not o.startswith('#'):
			return o
	return None



def process(lines, od, hinfo):
	stack = [('BODY', -1000)]	# [ (tag, indent), ... ]

	references = []
	eol = '\n'
	n = 0
	while 1:
		n += 1
		l = get_logical_line(lines)
		if l is None:
			break
		if l.lower().startswith('pre'):
			od.write('<pre>' + l[len('pre'):])
			while len(lines) > 0:
				l = lines.pop(0).rstrip()
				if l.lower().startswith('/pre'):
					break
				od.write(l + '\n')
			od.write("</pre>\n")
			if len(lines) == 0:
				break
			l = get_logical_line(lines)

		# print "#", l
		try:
			d = lineC(l)
		except ValueError, x:
			raise ValueError, "Bad Parse, line %d: %s" % (n, x)

		if d is None:
			# print "EMPTY"
			continue

		while d.indent <= stack[-1][1]:
			cltag, oindent = stack.pop(-1)
			od.write("</%s>%s" % (cltag, eol))

		if d.tag is not None:
			# print "args=", d.args
			some_refs, cltag = process_tag(d, od, hinfo, eol)
			if cltag is not None:
				stack.append((cltag, d.indent))
			references.extend( some_refs )

		elif d.txt is not None:
			od.write('%s%s' % (escape(d.txt), eol))

	while len(stack) > 0:
		od.write('</%s>%s' % (stack.pop(-1)[0], eol))

	return references


def swapend(a, e):
	t = a.rindex('.')
	return a[:t] + e


def aget(dic, alist):
	o = []
	for a in alist:
		for ac in [a, a.lower(), a.upper()]:
			if dic.has_key(ac):
				o.append('%s="%s"' % (ac, dic[ac]))
				break
	if len(o) == 0:
		return ''
	return " " + ' '.join(o)


def sw(s, prefix):
	return s.startswith(prefix) or s.startswith(prefix.lower()) or s.startswith(prefix.upper())

def dpre(s, prefix):
	return s[len(prefix):]

def agp(dic, prefix):
	lp = len(prefix)
	o = []
	for (k, v) in dic.items():
		if len(k) <= lp:
			continue
		if sw(k, prefix):
			o.append('%s="%s"' % (k[lp:], v))
	if len(o) == 0:
		return ''
	return " " + ' '.join(o)


def get(dic, key, dfl):
	for k in [key, key.lower(), key.upper()]:
		if dic.has_key(k):
			return dic[k]
	return dfl


def _header(thd, hinfo):
	thd.write( get(hinfo, 'DOCTYPE',
			'<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">'
			) + '\n'
		)
	thd.write('<html%s>\n' % aget(hinfo, ['lang']))
	thd.write("<head>\n")
	sty = get(hinfo, 'stylesheet', '')
	if sty != '':
		thd.write('<link rel="stylesheet" type="text/css" href="%s">' % sty)
	if hinfo.has_key('_mod_time'):
		thd.write('<meta http-equiv="Last-Modified" content="%s">\n'
				% time.asctime(time.gmtime(hinfo['_mod_time'])))
	for (k, v) in hinfo.items():
		if sw(k, 'M.'):
			thd.write('<meta name="%s" content="%s">\n' % (dpre(k, 'M.'), v))
		if sw(k, 'HTTP.'):
			thd.write('<meta http-equiv="%s" content="%s">\n' % (dpre(k, 'HTTP:'), v))
	thd.write("<title>%s</title>\n" % get(hinfo, 'TITLE', hinfo['filename']))
	thd.write("</head>\n")
	thd.write("<body%s>\n" % agp(hinfo, 'B.'))


def _common(lines, hinfo):
	return lines


def _footer(thd, hinfo):
	thd.write("</html>\n")


def readheader(fd, t):
	h = {'filename': t}
	while 1:
		l = fd.readline()
		if l is None:
			break
		l = l.rstrip()
		if l == '':
			break
		if l[0] == '#':
			continue
		a = l.split(None, 1)
		assert len(a) == 2
		h[a[0].strip()] = a[1].strip()
	return h


MTIME = 8

def needcopy(a, b, force=None):
	"""Do we need to reconstruct file b from
	file a?   Also reconstruct b if it is older
	than the date in force.
	"""
	assert os.access(a, os.F_OK + os.R_OK)
	ossa = os.stat(a)
	if not os.access(b, os.F_OK):
		return 1
	ossb = os.stat(b)
	if force is not None and ossb[MTIME] < force:
		return 1
	if ossb[MTIME] < ossa[MTIME]:
		return 1
	return 0


def file_only(url):
	"""Given a URL, just return the file part,
	not the target specifier inside the file."""

	a = url.split('#')
	if len(a) > 2:
		die.warn("URL has more than one '#': can't split into file and target.")
		return url
	return a[0]


def del_fin_sl(s):
	if s.endswith('/'):
		return s[:-1]
	return s


def go(env, force=None):
	htmlheader = env.get('header', _header)
	htmlfooter = env.get('footer', _footer)
	htmlcommon = env.get('common', _common)
	default_header = env.get('DEFAULT_HEADER', {})
	all = sets.Set()
	q = qfiles('.')
	for t in q:
		th = swapend(t, '.html')
		if not needcopy(t, th, force):
			print "# No need to process", t
			continue
		die.info("# processing:%s -> %s\n" % (t, th))
		fd = open(t, "r")
		# thd = open(th, "w")
		thd, thdo = g_pipe.popen2("/usr/bin/tidy", ['tidy', '-asxhtml', '-c', '-i', '-q', '-o', th])
		hinfo = default_header.copy()
		hinfo.update(readheader(fd, t))
		hinfo['_mod_time'] = os.fstat(fd.fileno()).st_mtime
		hinfo['_server_root'] = del_fin_sl(env.get('server', '.'))
		hinfo['_abs_root'] = del_fin_sl(env.get('server', '.'))
		hinfo['_rel_root'] = del_fin_sl(env.get('rel_root', '.'))
		if env.has_key('cwd'):
			hinfo['_cwd'] = env['cwd']
			hinfo['_local_root'] = env['cwd']
		htmlheader(thd, hinfo)
		lines = htmlcommon([x.rstrip() for x in fd.readlines()], hinfo)
		for reference in process(lines, thd, hinfo):
			all.add(reference)
		fd.close()
		htmlfooter(thd, hinfo)
		thd.close()

	return all



DEFF = 'headfoot.py'


def easygo():
	force = None
	env = {}
	cwd = os.getcwd().split('/') + ['.']
	for i in range(10):
		chkd = './' + ( '../' * i )
		chk = chkd + DEFF
		print "checking", chk
		if os.access(chk, os.R_OK):
			env['rel_root'] = chkd[:-1]
			env['cwd'] = '/'.join(cwd[-i-1:])
			execfile(chk, env)
			assert env.has_key('server')
			force = os.stat(chk)[MTIME]
			break
	return go(env, force)
	





if __name__ == '__main__':
	o = easygo()
	print "# Files to upload:"
	for t in o:
		print t



"""Solves various equations involving minimizing the
sum of absolute values of things.
"""


import math
from gmisclib import Num
from gmisclib import Numeric_gpk
from gmisclib import root

def solve1(y0, y1):
	"""We minimize the equation sum_over_i(abs(y0_i*(1-x) + y1_i*x))
	to find the best x.   Returns x.
	I don't know if this is strictly correct.
	"""
	n = len(y0)
	assert len(y1)==n
	yy0 = Num.asarray(y0)
	yy1 = Num.asarray(y1)
	slp = yy1-yy0
	zro = -yy0/slp
	i = Num.argsort(zro)
	zros = Num.take(zro, i, axis=0)
	slps = Num.absolute(Num.take(slp, i, axis=0))
	islps = Num.add.accumulate(slps)
	isl_tgt = 0.5*islps[-1]
	minidx = Num.searchsorted(islps, [isl_tgt])
	return zros[minidx]


def solve_fit(x, y, eps = 1e-7):
	"""Solves for the line y_hat = m*x + b that minimizes
	sum(abs(y - y_hat)).
	Algorithm from Numerical Recipes, Volume 2.
	Returns (mhat, bhat).
	"""
	assert eps > 0.0
	x = Num.asarray(x)
	y = Num.asarray(y)

	def b(x, y, m):
		return Numeric_gpk.N_median(y-m*x)

	mp2 = math.pi / 2.0

	def to_be_zeroed(theta, xy):
		x, y = xy
		if theta == mp2:
			return Num.sum(x * Num.sign(-x))
		if theta == -mp2:
			return Num.sum(x * Num.sign(x))

		m = math.tan(theta)
		bhat = b(x, y, m)
		return Num.sum(x * Num.sign(y - m*x - bhat) )

	mhat =  math.tan( root.root(to_be_zeroed, -mp2, mp2, (x, y),
					0.0,
					eps*Num.sum(Num.absolute(y))
					)
			)
	return (mhat, b(x, y, mhat))


def test1():
	assert solve1([-1], [1]) == 0
	assert solve1([0.5, 1], [0.1, 0]) == 1
	assert solve1([1.1, 0.6], [0, 0.1]) == 1


def test2():
	print solve_fit([0,1], [0,1])
	print solve_fit([0,1], [0,0.5])
	print solve_fit([0,1], [0,2])
	print solve_fit([0,1], [1,2])
	print solve_fit([math.exp(1),math.pi], [math.sqrt(2.0),1.0/97.0])

if __name__ == '__main__':
	test1()
	test2()



import Num
import types

class multivariate_normal:
	__doc__ = """multivariate_normal(mean, cov) or multivariate_normal(mean, cov, [m, n, ...])
          returns an array containing multivariate normally distributed random numbers
          with specified mean and covariance.

          mean must be a 1 dimensional array. cov must be a square two dimensional
          array with the same number of rows and columns as mean has elements.

          The first form returns a single 1-D array containing a multivariate
          normal.

          The second form returns an array of shape (m, n, ..., cov.shape[0]).
          In this case, output[i,j,...,:] is a 1-D array containing a multivariate
          normal."""

	def __init__(self, mean, cov):
		cov = Num.asarray(cov)
       		mean = Num.array(mean, copy=True)
       		# Check preconditions on arguments
       		if len(mean.shape) != 1:
              		raise TypeError, "mean must be 1 dimensional."
       		if (len(cov.shape) != 2) or (cov.shape[0] != cov.shape[1]):
              		raise TypeError, "cov must be 2 dimensional and square."
       		if mean.shape[0] != cov.shape[0]:
              		raise TypeError, "mean and cov must have same length."
       		u, s, self.v = Num.LA.singular_value_decomposition(cov)
		self.ss = Num.sqrt(s)
		self.mean = Num.array(mean, copy=True)	# Copy, so no one changes it
						# underneath you.


	def sample(self, shape=[]):
       		# Compute shape of output
       		if isinstance(shape, types.IntType):
			final_shape = [shape]
		else:
       			final_shape = list(shape[:])
       		final_shape.append(self.mean.shape[0])
       		# Create a matrix of independent standard normally distributed random
       		# numbers. The matrix has rows with the same length as mean and as
       		# many rows are necessary to form a matrix of shape final_shape.
       		x = Num.RA.standard_normal(Num.multiply.reduce(final_shape))
       		x.shape = (Num.multiply.reduce(final_shape[0:-1]), final_shape[-1])
       		# Transform matrix of standard normals into matrix where each row
       		# contains multivariate normals with the desired covariance.
       		# Compute A such that matrixmultiply(transpose(A),A) == cov.
       		# Then the matrix products of the rows of x and A has the desired
       		# covariance. Note that sqrt(s)*v where (u,s,v) is the singular value
       		# decomposition of cov is such an A.
       		x = Num.matrixmultiply(x*self.ss, self.v)
       		# The rows of x now have the correct covariance but mean 0. Add
       		# mean to each row. Then each row will have mean mean.
       		Num.add(self.mean, x, x)
       		x.shape = final_shape
       		return x

def test():
	import sys
	N = 30000
	cov = Num.array([[1, 0.1, -0.2], [0.1, 1.1, 0.2], [-0.2, 0.2, 2.0]])
	x = multivariate_normal(Num.zeros((3,), Num.Float),
				cov)
	s = Num.zeros((3,3), Num.Float)
	for i in range(N):
		z = x.sample()
		if i%300 == 0:
			sys.stdout.writelines('.')
		s += Num.outerproduct(z, z)
	print
	print cov
	print s/N

if __name__ == '__main__':
	test()



import os

class writer(object):
	def comments(self, comments):
		"""Add comments to the data file.
		Comments can appear anywhere.
		"""
		for comment in comments:
			self.comment(comment)

	def comment(self, comment):
		raise RuntimeError, "Virtual Function"

	def header(self, k, v):
		raise RuntimeError, "Virtual Function"
	
	def headers(self, h):
		hdritems = h.items()
		hdritems.sort()
		for (k, v) in hdritems:
			self.header(k, v)


	def __init__(self, fd):
		self.fd = fd

	def data(self, dataset):
		"""Write a series of lines to the output file."""
		for datum in dataset:
			self.datum(datum)
	
	def extend(self, d):
		self.data(d)

	def append(self, d):
		self.datum(d)

	def datum(self, data_item):
		raise RuntimeError, "Virtual Function"
	

	def flush(self):
		if hasattr(self.fd, 'flush'):
			self.fd.flush()
			os.fsync(self.fd.fileno())

	def close(self):
		# Don't explicitly close it, as self.fd
		# could be sys.stdout.   We just de-refererence
		# it, so that it automatically closes if nothing
		# else keeps it open.
		if self.fd:
			self.flush()
		self.fd = None



class null_writer(writer):
	def comments(self, comments):
		pass

	def comment(self, comment):
		pass

	def header(self, k, v):
		pass
	
	def headers(self, h):
		pass

	def __init__(self):
		pass

	def data(self, dataset):
		pass
	
	def datum(self, data_item):
		pass
	
	def __del__(self):
		self.close()

	def flush(self):
		pass

	def close(self):
		pass



"""This is a compatibility module, allowing python to work
with either scipy, Numeric, or numarray.
It tries to import them, in that order, only reporting
an error if none are available.

Note that this does not pretend to solve all compatibility
problems; it just tries importing all three, so you can only
count on the lowest common denominator.
"""

class _delayed_import(object):
	def __init__(self, name, aliases = []):
		self.name = name
		self.mod = None
		self.aliases = aliases

	def __getattribute__(self, aname):
		mod = object.__getattribute__(self, 'mod')
		if mod is None:
			mname = object.__getattribute__(self, 'name')
			mod = __import__(mname)
			a = mname.split('.')
			for dot in a[1:]:
				mod = getattr(mod, dot)
			self.mod = mod
			for (nn, on) in object.__getattribute__(self, 'aliases'):
				setattr(mod, nn, getattr(mod, on))
		return getattr(mod, aname)


array = None
try:
	from numpy import *
	LA = _delayed_import('numpy.linalg',
				[
				('linear_least_squares', 'lstsq'),
				('singular_value_decomposition', 'svd')
				]
				)
	RA = _delayed_import('numpy.random')
	FFT = _delayed_import('numpy.fft',
				[
				('inverse_fft', 'ifft'), ('real_fft', 'rfft'),
				('inverse_real_fft', 'irfft')
				]
				)
	Float = double
	Int = int0
	Int32 = int32
	Int16 = int16
	Int8 = int8
	Complex = complex_
	arrayrange = arange
	matrixmultiply = dot
	outerproduct = outer
	NewAxis = newaxis
except ImportError:
	pass

if array is None:
	try:
		from Numeric import *	# This normally imports array
		RA = _delayed_import('RandomArray')
		LA = _delayed_import('LinearAlgebra')
		FFT = _delayed_import('FFT')
	except ImportError:
		pass



if array is None:
	try:
		from numarray import *
	except ImportError:
		raise ImportError, "No module named either numpy, Numeric or numarray"

assert FFT
assert LA
assert RA





"""This rotates a matrix by multiplying with a unitary matrix
so that the resulting elements are either nearly zero or relatively large.
It minimizes sum of |x_ij|*log(|x_ij|), i.e. the entropy (sortof).
"""

import math
from gmisclib import Num
from gmisclib import mcmc
from gmisclib import mcmc_helper
from gmisclib import gpkmisc


def make_min_entropy(x, extra_entropy=0.0):
	"""extra_entropy is a vector that helps choose what to optimize."""
	x = Num.asarray(x, Num.Float)
	print 'x.shape=', x.shape
	ndim, nvec = x.shape

	def unitary(p):
		assert p.shape[0] == nvec**2
		pm = Num.reshape(p, (nvec, nvec))
		psym = Num.dot(pm, Num.transpose(pm))
		evalues, umat = Num.LA.eigh(psym)
		return umat

	def rotated(xt, p):
		frot = Num.dot(unitary(p), xt)
		assert frot.shape == xt.shape
		return frot

	def fom(p, c):
		xt, nvec, extra_entropy, G = c
		frot = rotated(xt, p)
		assert frot.shape[0] == nvec
		# print 'frot=', frot
		frotfom = Num.absolute(frot)
		frotn = frotfom/Num.sum(frotfom, axis=1)[:,Num.NewAxis]
		assert abs(Num.sum(frotn[0])-1.0) < 0.01
		nege = Num.sum(frotn*(Num.log(frotn)-extra_entropy), axis=1)
		assert nege.shape == (nvec,)
		print 'fom=', Num.sum(nege)
		return G*Num.sum(nege)

	def fixer(p, c):
		Num.divide(p, math.sqrt(Num.average(p**2)), p)
		return p

	prmvec = Num.RA.normal(0.0, 1.0, (nvec**2,))
	stpr = mcmc.bootstepper(fom, [prmvec], v=gpkmisc.make_diag(prmvec**2),
					c = (Num.transpose(x), nvec, extra_entropy, 20.0),
					fixer=fixer
					)
	mcmch = mcmc_helper.stepper(stpr)
	mcmch.run_to_bottom()
	stpr.T = 0.3
	mcmch.run_to_bottom()
	stpr.T = 0.1
	mcmch.run_to_bottom()
	prms = stpr.current().prms()
	return (Num.transpose(rotated(Num.transpose(x), prms)),
		unitary(prms)
		)


def test2():
	x = [[1.0, 1.0], [-1.0, 1.0]]
	y,u = make_min_entropy(x)
	assert abs(Num.sum(u**2) - u.shape[0]) < 0.001
	print 'y=', y
	ay = Num.absolute(y)
	assert Num.sum(Num.less(ay, 0.001)) == 2
	r2 = math.sqrt(2.0)
	assert Num.sum(Num.less(Num.absolute(ay-r2), 0.001)) == 2

def test3():
	x = [[0, 0, 1], [1.0, -1.0, 0], [1.0, 1.0, 0]]
	y,u = make_min_entropy(x)
	assert abs(Num.sum(u**2) - u.shape[0]) < 0.001
	print 'y=', y
	ay = Num.absolute(y)
	assert Num.sum(Num.less(ay, 0.01)) == 6
	r2 = math.sqrt(2.0)
	assert Num.sum(Num.less(Num.absolute(ay-r2), 0.01)) == 2
	assert Num.sum(Num.less(Num.absolute(ay-1), 0.01)) == 1


if __name__ == '__main__':
	test2()
	test3()




"""This reads MLF (Master Label Files) for/from the HTK speech recognition
toolkit.
"""

import os
import glob
from gmisclib import die
from gmisclib.xwaves_errs import *

TIME_QUANTUM = 1e-7
# At 11025 Hz sampling rate.
# TIME_QUANTUM = 1e-7 * (11025.0/11000.0)

class ReferencedFileNotFound(Error):
	def __init__(self, *s):
		Error.__init__(self, *s)


def _findfile(f, postfix, verbose):
	"""Find a file.   Starting with a glob expression C{f}, strip off
	any .something suffix, then add on C{postfix}.
	If that glob evaluates to a file, return (d,f)
	where C{d} is the directory and C{f} is the filename (with postfix removed).
	Postfix would normally start with '.'.
	At the end, os.path.join(d, f) + postfix will name a file.
	"""
	if not f:
		return (None, None)
	d0, f0 = os.path.split(f)
	f0 = os.path.splitext(f0)[0]
	fx = os.path.join(d0, f0) + postfix
	if verbose:
		die.info("HTK_MLF_io._findfile: glob=%s" % fx)
	gl = glob.glob(fx)
	if verbose:
		die.info("HTK_MLF_io._findfile globs=%s" % ','.join(gl))
	if len(gl) == 0:
		raise ReferencedFileNotFound, fx
	assert len(gl) == 1, 'Too many alternatives for %s: %d' % (fx, len(gl))
	d1, f1 = os.path.split(gl[0])
	# assert f1.endswith(postfix)
	f1 = os.path.splitext(f1)[0]
	# f1 = f1[:-len(postfix)]
	if verbose:
		die.info("HTK_MLF_io._findfile d1=%s d1=%s" % (d1, f1))
	return (d1, f1)


def parse_label_line(s, tq):
	a = s.strip().split()
	la = len(a)
	if la == 1:
		if a[0] == '///':
			die.die("Sorry! cannot handle /// in MLF.")
		return a[0]	# name
	elif tq is None:
		raise ValueError, "File data needs time_quantum to be a number."
	elif la >= 3:
		tmp = (float(a[0])*tq, float(a[1])*tq, a[2])
		if la == 3:
			return tmp	# (start, end, name)
		tmp = list(tmp)
		a = a[3:]
		while a:
			tmp.append(float(a.pop(0)))	# Score or Auxscore
			if not a:
				break
			tmp.append(a.pop(0))	# Auxname
		return tuple(tmp)
	elif la == 2:
		return (float(a[0])*tq, a[1])	# (start, name)
	raise BadFileFormatError


def _get_symbols(fd, time_quantum):
        sym = []
        while True:
                s = fd.readline()
                if s == '':
			break
                s = s.rstrip('\r\n')
                if s == '.':
                        break
		elif s != '':
                	sym.append( parse_label_line(s, time_quantum) )
        return sym


def readone(mlf_efn, postfix='.wav', datapath='.', strict=True,
		findfile=True, pathedit=None, time_quantum=TIME_QUANTUM,
		verbose=False):
	"""Read a single set of labels from a MLF file.
	You specify the labels as part of the extended filename, like this:
	name_of_MLF_file:name_of_labels'.    The function returns
	only a single value and raises an exception if the  extended
	filename is ambiguous.
	@type mlf_efn: string in the form "F:S"
	@rtype dict()
	@return: a dictionary that describes the labels as per
		L{readiter}.
	"""
	filename, subname = mlf_efn.split(':')
	candidate = None
	for x in readiter(filename, postfix=postfix, datapath=datapath,
			strict=strict, findfile=findfile,
			pathedit=pathedit, time_quantum=time_quantum,
			verbose=verbose):
		if subname in x['filespec']:
			if candidate is not None:
				raise ValueError, "Not unique: %s in %s" % (subname, filename)
			candidate = x
	return candidate


def readiter(mlf_fn, postfix='.wav', datapath='.', strict=True,
		findfile=True, pathedit=None,
		time_quantum=TIME_QUANTUM,
		verbose=False):
	"""Read a MLF file (relevant to HTK took kit).
	Returns [ {'filespec':path, 'd': d, 'f': f, 'symbols': zzz}, ... ]
	where 'd' and 'f' are only present if findfile==True.
	Datapath and pathedit are ways to deal with the
	situation where the MLF file has been moved, or (for other reasons)
	the filenames in the MLF file don't point to the actual data.

	"""
	if pathedit is None:
		pathedit = os.path.join

	dmlf, fmlf = os.path.split(mlf_fn)

	try:
		fd = open(mlf_fn, 'r')
	except IOError, x:
		raise NoSuchFileError(*(x.args))
	l = fd.readline()
	assert l=='#!MLF!#\n', 'l=%s' % l
	i = 0
	while True:
		f = fd.readline()
		if f == '':
			break
		f = f.strip()
		if f == '':
			continue
		if f.startswith('"') and f.endswith('"'):
			f = f[1:-1]
		fspec = f
		rv = {'filespec': fspec, 'i': i}
		if findfile:
			if verbose:
				die.info("dmlf=%s; datapath=%s; f=%s" % (dmlf, datapath, f))
			try:
				d1, f1 = _findfile(pathedit(dmlf,datapath,f), postfix, verbose)
			except ReferencedFileNotFound, x:
				if strict:
					raise
				else:
					die.warn('No such file: %s from %s' % (x, fspec))
					_get_symbols(fd, None)
					continue
			else:
				rv['d'] = d1
				rv['f'] = f1
		rv['symbols'] = _get_symbols(fd, time_quantum)
		yield rv
		i += 1


def read(mlf_fn, **kw):
	return list( readiter(mlf_fn, **kw) )


class writer(object):
	def __init__(self, mlf_fd, time_quantum=TIME_QUANTUM):
		assert time_quantum > 0.0
		self.fd = mlf_fd
		self.fd.writelines('#!MLF!#\n')
		self.nchunks = 0
		self.scale = 1.0/time_quantum

	def chunk(self, filespec, data):
		if self.nchunks > 0:
			self.fd.writelines('.\n')
		self.fd.writelines( [ '"%s"\n' % filespec,
					'\n'.join(data), '\n'
					]
				)
		self.nchunks += 1
		self.fd.flush()

	def threecol(self, filespec, tcdata):
		d = [ '%d %d %s' % (int(round(t0*self.scale)),
					int(round(te*self.scale)),
					lbl) 
			for (t0, te, lbl) in tcdata
			]
		self.chunk(filespec, d)

	def close(self):
		self.fd.writelines('\n')
		self.fd.flush()
		os.fsync(self.fd.fileno())
		self.fd = None	# Normally, this will close the file.

	def __del__(self):
		if self.fd is not None:
			self.close()


if __name__ == '__main__':
	import sys
	DATAPATH = None
	arglist = sys.argv[1:]
	while arglist and arglist[0].startswith('-'):
		arg = arglist.pop(0)
		if arg == '-datapath':
			DATAPATH = arglist.pop(0)
		else:
			die.die('Unrecognized argument: %s' % arg)
	for tmp in readiter(arglist[0], datapath=DATAPATH,
				findfile=(DATAPATH is not None)
				):
		print '[', tmp['filespec'], tmp.get('d', ''), tmp.get('f', ''), ']'
		for tmps in tmp['symbols']:
			if isinstance(tmps, tuple):
				print ' '.join([str(q) for q in tmps])
			else:
				print tmps
